@@ -1,6 +1,6 @@
 use chrono::prelude::Utc;
 use anyhow::{Result, Context};
-use std::time::Instant;
+use std::time::{Instant, Duration};
 use zookeeper::{ZkResult, ZooKeeper, CreateMode, ZooKeeperExt};
 use std::sync::{Arc, Mutex};
 use log::{debug, warn, trace, error};
@@ -16,7 +16,7 @@ pub use path::*;
 
 
 const ZKMQ_CONSUMER_CONSUMPTION_LIMIT: usize = 12;
-
+const ZKMQ_CONSUMER_LATCH_TIMEOUT: u64 = 3000;
 
 #[derive(Clone, Derivative)]
 #[derivative(Debug)]
@@ -65,57 +65,25 @@ impl ZkMQBuilder {
     }
 
     fn build_tree(&self, nodes: Vec<&str>) -> ZkResult<String> {
-
-        fn create_node(zk: Arc<ZooKeeper>, path: &str) -> ZkResult<String> {
-            if zk.exists(path, false)?.is_none() {
-                debug!("creating {}", path);
-                zk.create(path, vec![], zookeeper::Acl::open_unsafe().clone(), zookeeper::CreateMode::Container)
-            } else {
-                trace!("{} already exists", path);
-                Ok("".to_string())
-            }
-        }
-
-        // create base first
-        let paths = self.dir.split('/').collect::<Vec<&str>>();
-        let mut working_path = "".to_string();
-
-        for path in paths.iter() {
-            if path == &"" { continue; }
-            working_path.push('/');
-            working_path.push_str(path);
-
-           create_node(self.zk.clone(), working_path.as_str())?;
-        }
-
-        // now create nodes in the base
         for node in nodes {
-            create_node(self.zk.clone(), format!("{}/{}", &working_path, node).as_str())?;
+            self.zk.ensure_path(format!("{}/{}", &self.dir, node).as_str())?;
         }
-
         Ok("".to_string())
     }
 
     pub fn build(self) -> Result<ZkMQ> {
-
         if !self.producer_started && !self.consumer_started {
             warn!("building ZkMQ object that cannot consume nor produce messages");
         }
 
-        let id = self.id.clone().unwrap_or_else(|| uuid::Uuid::new_v4().to_string());
-
         self.build_tree(["queue", "task", "conf", "lock"].to_vec())?;
-        let dir = ZkPath::new(self.dir.clone());
 
-        let z = ZkMQ {
-            id,
-            dir,
+        Ok(ZkMQ {
+            id: self.id.clone().unwrap_or_else(|| uuid::Uuid::new_v4().to_string()),
+            dir: ZkPath::new(self.dir.clone()),
             config: self,
             cache: Arc::new(Mutex::new(lru::LruCache::new(1024)))
-        };
-
-        Ok(z)
-
+        })
     }
 }
 
@@ -261,7 +229,6 @@ impl ZkMQ {
         trace!("{:?}", &message);
 
         Ok(message)
-
     }
 
     /// Consume messages from the Queue
@@ -322,13 +289,17 @@ impl ZkMQ {
                 continue;
             }
 
-            // otherwise, wait until the handler is called and try this again
-            let _ = latch.1.recv().unwrap();
+            // wait for the latch to trip, or `ZKMQ_CONSUMER_LATCH_TIMEOUT` ms. whatever happens first
+            // timeout was added to handle the case where a consumer starts with an empty queue. In that case, the latch
+            // would never trip.
+            let _ = latch.1.recv_timeout(Duration::from_millis(ZKMQ_CONSUMER_LATCH_TIMEOUT)).unwrap();
         }
     }
 
 
-    // Producer Functions
+    /// Produce a message into the current queue.
+    ///
+    /// Error is returned if the operation fails in any way.
     pub fn produce<P: Into<Vec<u8>>>(&self, payload: P) -> Result<()> {
         if !self.config.producer_started {
             anyhow::bail!("producer is not started");
@@ -343,17 +314,16 @@ impl ZkMQ {
         self.product_adv(msg)
     }
 
+    /// Produce a message a raw `ZkMQMessage` into the current queue
     pub fn product_adv(&self, message: ZkMQMessage) -> Result<()> {
         if !self.config.producer_started {
             anyhow::bail!("producer is not started");
         }
-
         let start = Instant::now();
 
         debug!(target: &*format!("zkmq::{}", self.id), "enqueuing message {}", &message.id);
 
         let path = self.dir.join("task").join(&message.id);
-
         let tx: Result<()> = {
             let _ = self.create_znode(path.clone(), vec![])?;
             let _ = self.create_znode(path.join("filters"), vec![])?;
